==> typescript-browser-0.0.24-18.tgz: package/src/typescript/language/index.ts <==
/// <reference types='monaco-editor/monaco'/>

export const JAVASCRIPT_LANGUAGE_ID = 'javascript';
export const JAVASCRIPT_LANGUAGE_NAME = 'JavaScript';

export const TYPESCRIPT_LANGUAGE_ID = 'typescript';
export const TYPESCRIPT_LANGUAGE_NAME = 'TypeScript';

export const genericEditConfiguration: monaco.languages.LanguageConfiguration = {
    wordPattern: /(-?\d*\.\d\w*)|([^\`\~\!\@\#\%\^\&\*\(\)\-\=\+\[\{\]\}\\\|\;\:\'\"\,\.\<\>\/\?\s]+)/g,
    comments: {
        lineComment: '//',
        blockComment: ['/*', '*/'],
    },

    brackets: [
        ['{', '}'],
        ['[', ']'],
        ['(', ')'],
    ],

    onEnterRules: [
        {
            // e.g. /** | */
            beforeText: /^\s*\/\*\*(?!\/)([^\*]|\*(?!\/))*$/,
            afterText: /^\s*\*\/$/,
            action: { indentAction: monaco.languages.IndentAction.IndentOutdent, appendText: ' * ' },
        },
        {
            // e.g. /** ...|
            beforeText: /^\s*\/\*\*(?!\/)([^\*]|\*(?!\/))*$/,
            action: { indentAction: monaco.languages.IndentAction.None, appendText: ' * ' },
        },
        {
            // e.g.  * ...|
            beforeText: /^(\t|(\ \ ))*\ \*(\ ([^\*]|\*(?!\/))*)?$/,
            action: { indentAction: monaco.languages.IndentAction.None, appendText: '* ' },
        },
        {
            // e.g.  */|
            beforeText: /^(\t|(\ \ ))*\ \*\/\s*$/,
            action: { indentAction: monaco.languages.IndentAction.None, removeText: 1 },
        },
    ],

    autoClosingPairs: [
        { open: '{', close: '}' },
        { open: '[', close: ']' },
        { open: '(', close: ')' },
        { open: '"', close: '"', notIn: ['string'] },
        { open: '\'', close: '\'', notIn: ['string', 'comment'] },
        { open: '`', close: '`', notIn: ['string', 'comment'] },
        { open: '/**', close: ' */', notIn: ['string'] },
    ],
};

import ts = require('monaco-typescript/release/lib/typescriptServices');

export enum Language {
    TypeScript,
    EcmaScript5,
}

export function createTokenizationSupport(language): monaco.languages.TokensProvider {

    const classifier = ts.createClassifier(),
        bracketTypeTable = language === Language.TypeScript ? tsBracketTypeTable : jsBracketTypeTable,
        tokenTypeTable = language === Language.TypeScript ? tsTokenTypeTable : jsTokenTypeTable;

    return {
        getInitialState: () => new State(language, ts.EndOfLineState.None, false),
        tokenize: (line, state) => tokenize(bracketTypeTable, tokenTypeTable, classifier, <State>state, line),
    };
}

class State implements monaco.languages.IState {

    public language: Language;
    public eolState: /*ts.EndOfLineState*/ any;
    public inJsDocComment: boolean;

    constructor(language: Language, eolState: /*ts.EndOfLineState*/ any, inJsDocComment: boolean) {
        this.language = language;
        this.eolState = eolState;
        this.inJsDocComment = inJsDocComment;
    }

    public clone(): State {
        return new State(this.language, this.eolState, this.inJsDocComment);
    }

    public equals(other: monaco.languages.IState): boolean {
        if (other === this) {
            return true;
        }
        if (!other || !(other instanceof State)) {
            return false;
        }
        if (this.eolState !== (<State>other).eolState) {
            return false;
        }
        if (this.inJsDocComment !== (<State>other).inJsDocComment) {
            return false;
        }
        return true;
    }
}

function tokenize(bracketTypeTable: { [i: number]: string }, tokenTypeTable: { [i: number]: string },
    classifier: /*ts.Classifier*/ any, state: State, text: string): monaco.languages.ILineTokens {

    // Create result early and fill in tokens
    const ret = {
        tokens: <monaco.languages.IToken[]>[],
        endState: new State(state.language, ts.EndOfLineState.None, false),
    };

    function appendFn(startIndex: number, type: string): void {
        if (ret.tokens.length === 0 || ret.tokens[ret.tokens.length - 1].scopes !== type) {
            ret.tokens.push({
                startIndex: startIndex,
                scopes: type,
            });
        }
    }

    const isTypeScript = state.language === Language.TypeScript;

    // shebang statement, #! /bin/node
    if (!isTypeScript && checkSheBang(0, text, appendFn)) {
        return ret;
    }

    const result = classifier.getClassificationsForLine(text, state.eolState, true);
    let offset = 0;

    ret.endState.eolState = result.finalLexState;
    ret.endState.inJsDocComment = result.finalLexState === ts.EndOfLineState.InMultiLineCommentTrivia && (state.inJsDocComment || /\/\*\*.*$/.test(text));

    for (let entry of result.entries) {

        let type: string;

        if (entry.classification === ts.TokenClass.Punctuation) {
            // punctions: check for brackets: (){}[]
            const ch = text.charCodeAt(offset);
            type = bracketTypeTable[ch] || tokenTypeTable[entry.classification];
            appendFn(offset, type);

        } else if (entry.classification === ts.TokenClass.Comment) {
            // comments: check for JSDoc, block, and line comments
            if (ret.endState.inJsDocComment || /\/\*\*.*\*\//.test(text.substr(offset, entry.length))) {
                appendFn(offset, isTypeScript ? 'comment.doc.ts' : 'comment.doc.js');
            } else {
                appendFn(offset, isTypeScript ? 'comment.ts' : 'comment.js');
            }
        } else {
            // everything else
            appendFn(offset,
                tokenTypeTable[entry.classification] || '');
        }

        offset += entry.length;
    }

    return ret;
}

interface INumberStringDictionary {
    [idx: number]: string;
}

const tsBracketTypeTable: INumberStringDictionary = Object.create(null);
tsBracketTypeTable['('.charCodeAt(0)] = 'delimiter.parenthesis.ts';
tsBracketTypeTable[')'.charCodeAt(0)] = 'delimiter.parenthesis.ts';
tsBracketTypeTable['{'.charCodeAt(0)] = 'delimiter.bracket.ts';
tsBracketTypeTable['}'.charCodeAt(0)] = 'delimiter.bracket.ts';
tsBracketTypeTable['['.charCodeAt(0)] = 'delimiter.array.ts';
tsBracketTypeTable[']'.charCodeAt(0)] = 'delimiter.array.ts';

const tsTokenTypeTable: INumberStringDictionary = Object.create(null);
tsTokenTypeTable[ts.TokenClass.Identifier] = 'identifier.ts';
tsTokenTypeTable[ts.TokenClass.Keyword] = 'keyword.ts';
tsTokenTypeTable[ts.TokenClass.Operator] = 'delimiter.ts';
tsTokenTypeTable[ts.TokenClass.Punctuation] = 'delimiter.ts';
tsTokenTypeTable[ts.TokenClass.NumberLiteral] = 'number.ts';
tsTokenTypeTable[ts.TokenClass.RegExpLiteral] = 'regexp.ts';
tsTokenTypeTable[ts.TokenClass.StringLiteral] = 'string.ts';

const jsBracketTypeTable: INumberStringDictionary = Object.create(null);
jsBracketTypeTable['('.charCodeAt(0)] = 'delimiter.parenthesis.js';
jsBracketTypeTable[')'.charCodeAt(0)] = 'delimiter.parenthesis.js';
jsBracketTypeTable['{'.charCodeAt(0)] = 'delimiter.bracket.js';
jsBracketTypeTable['}'.charCodeAt(0)] = 'delimiter.bracket.js';
jsBracketTypeTable['['.charCodeAt(0)] = 'delimiter.array.js';
jsBracketTypeTable[']'.charCodeAt(0)] = 'delimiter.array.js';

const jsTokenTypeTable: INumberStringDictionary = Object.create(null);
jsTokenTypeTable[ts.TokenClass.Identifier] = 'identifier.js';
jsTokenTypeTable[ts.TokenClass.Keyword] = 'keyword.js';
jsTokenTypeTable[ts.TokenClass.Operator] = 'delimiter.js';
jsTokenTypeTable[ts.TokenClass.Punctuation] = 'delimiter.js';
jsTokenTypeTable[ts.TokenClass.NumberLiteral] = 'number.js';
jsTokenTypeTable[ts.TokenClass.RegExpLiteral] = 'regexp.js';
jsTokenTypeTable[ts.TokenClass.StringLiteral] = 'string.js';

function checkSheBang(deltaOffset: number, line: string, appendFn: (startIndex: number, type: string) => void): boolean {
    if (line.indexOf('#!') === 0) {
        appendFn(deltaOffset, 'comment.shebang');
        return true;
    }
    return false;
}

==> typescript-browser-0.0.24-40.tgz: package/src/typescript/language/index.ts <==
import {
    LanguageConfiguration, TokensProvider,
    IState, ILineTokens, IToken, IndentAction,
} from '../../interfaces';
export const JAVASCRIPT_LANGUAGE_ID = 'javascript';
export const JAVASCRIPT_LANGUAGE_NAME = 'JavaScript';

export const TYPESCRIPT_LANGUAGE_ID = 'typescript';
export const TYPESCRIPT_LANGUAGE_NAME = 'TypeScript';

export const genericEditConfiguration: LanguageConfiguration = {
    wordPattern: /(-?\d*\.\d\w*)|([^\`\~\!\@\#\%\^\&\*\(\)\-\=\+\[\{\]\}\\\|\;\:\'\"\,\.\<\>\/\?\s]+)/g,
    comments: {
        lineComment: '//',
        blockComment: ['/*', '*/'],
    },

    brackets: [
        ['{', '}'],
        ['[', ']'],
        ['(', ')'],
    ],

    onEnterRules: [
        {
            // e.g. /** | */
            beforeText: /^\s*\/\*\*(?!\/)([^\*]|\*(?!\/))*$/,
            afterText: /^\s*\*\/$/,
            action: { indentAction: IndentAction.IndentOutdent, appendText: ' * ' },
        },
        {
            // e.g. /** ...|
            beforeText: /^\s*\/\*\*(?!\/)([^\*]|\*(?!\/))*$/,
            action: { indentAction: IndentAction.None, appendText: ' * ' },
        },
        {
            // e.g.  * ...|
            beforeText: /^(\t|(\ \ ))*\ \*(\ ([^\*]|\*(?!\/))*)?$/,
            action: { indentAction: IndentAction.None, appendText: '* ' },
        },
        {
            // e.g.  */|
            beforeText: /^(\t|(\ \ ))*\ \*\/\s*$/,
            action: { indentAction: IndentAction.None, removeText: 1 },
        },
    ],

    autoClosingPairs: [
        { open: '{', close: '}' },
        { open: '[', close: ']' },
        { open: '(', close: ')' },
        { open: '"', close: '"', notIn: ['string'] },
        { open: '\'', close: '\'', notIn: ['string', 'comment'] },
        { open: '`', close: '`', notIn: ['string', 'comment'] },
        { open: '/**', close: ' */', notIn: ['string'] },
    ],
};

import ts = require('monaco-typescript/release/lib/typescriptServices');

export enum Language {
    TypeScript,
    EcmaScript5,
}

export function createTokenizationSupport(language): TokensProvider {

    const classifier = ts.createClassifier(),
        bracketTypeTable = language === Language.TypeScript ? tsBracketTypeTable : jsBracketTypeTable,
        tokenTypeTable = language === Language.TypeScript ? tsTokenTypeTable : jsTokenTypeTable;

    return {
        getInitialState: () => new State(language, ts.EndOfLineState.None, false),
        tokenize: (line, state) => tokenize(bracketTypeTable, tokenTypeTable, classifier, <State>state, line),
    };
}

class State implements IState {

    public language: Language;
    public eolState: /*ts.EndOfLineState*/ any;
    public inJsDocComment: boolean;

    constructor(language: Language, eolState: /*ts.EndOfLineState*/ any, inJsDocComment: boolean) {
        this.language = language;
        this.eolState = eolState;
        this.inJsDocComment = inJsDocComment;
    }

    public clone(): State {
        return new State(this.language, this.eolState, this.inJsDocComment);
    }

    public equals(other: IState): boolean {
        if (other === this) {
            return true;
        }
        if (!other || !(other instanceof State)) {
            return false;
        }
        if (this.eolState !== (<State>other).eolState) {
            return false;
        }
        if (this.inJsDocComment !== (<State>other).inJsDocComment) {
            return false;
        }
        return true;
    }
}

function tokenize(bracketTypeTable: { [i: number]: string }, tokenTypeTable: { [i: number]: string },
    classifier: /*ts.Classifier*/ any, state: State, text: string): ILineTokens {

    // Create result early and fill in tokens
    const ret = {
        tokens: <IToken[]>[],
        endState: new State(state.language, ts.EndOfLineState.None, false),
    };

    function appendFn(startIndex: number, type: string): void {
        if (ret.tokens.length === 0 || ret.tokens[ret.tokens.length - 1].scopes !== type) {
            ret.tokens.push({
                startIndex: startIndex,
                scopes: type,
            });
        }
    }

    const isTypeScript = state.language === Language.TypeScript;

    // shebang statement, #! /bin/node
    if (!isTypeScript && checkSheBang(0, text, appendFn)) {
        return ret;
    }

    const result = classifier.getClassificationsForLine(text, state.eolState, true);
    let offset = 0;

    ret.endState.eolState = result.finalLexState;
    ret.endState.inJsDocComment = result.finalLexState === ts.EndOfLineState.InMultiLineCommentTrivia && (state.inJsDocComment || /\/\*\*.*$/.test(text));

    for (let entry of result.entries) {

        let type: string;

        if (entry.classification === ts.TokenClass.Punctuation) {
            // punctions: check for brackets: (){}[]
            const ch = text.charCodeAt(offset);
            type = bracketTypeTable[ch] || tokenTypeTable[entry.classification];
            appendFn(offset, type);

        } else if (entry.classification === ts.TokenClass.Comment) {
            // comments: check for JSDoc, block, and line comments
            if (ret.endState.inJsDocComment || /\/\*\*.*\*\//.test(text.substr(offset, entry.length))) {
                appendFn(offset, isTypeScript ? 'comment.doc.ts' : 'comment.doc.js');
            } else {
                appendFn(offset, isTypeScript ? 'comment.ts' : 'comment.js');
            }
        } else {
            // everything else
            appendFn(offset,
                tokenTypeTable[entry.classification] || '');
        }

        offset += entry.length;
    }

    return ret;
}

interface INumberStringDictionary {
    [idx: number]: string;
}

const tsBracketTypeTable: INumberStringDictionary = Object.create(null);
tsBracketTypeTable['('.charCodeAt(0)] = 'delimiter.parenthesis.ts';
tsBracketTypeTable[')'.charCodeAt(0)] = 'delimiter.parenthesis.ts';
tsBracketTypeTable['{'.charCodeAt(0)] = 'delimiter.bracket.ts';
tsBracketTypeTable['}'.charCodeAt(0)] = 'delimiter.bracket.ts';
tsBracketTypeTable['['.charCodeAt(0)] = 'delimiter.array.ts';
tsBracketTypeTable[']'.charCodeAt(0)] = 'delimiter.array.ts';

const tsTokenTypeTable: INumberStringDictionary = Object.create(null);
tsTokenTypeTable[ts.TokenClass.Identifier] = 'identifier.ts';
tsTokenTypeTable[ts.TokenClass.Keyword] = 'keyword.ts';
tsTokenTypeTable[ts.TokenClass.Operator] = 'delimiter.ts';
tsTokenTypeTable[ts.TokenClass.Punctuation] = 'delimiter.ts';
tsTokenTypeTable[ts.TokenClass.NumberLiteral] = 'number.ts';
tsTokenTypeTable[ts.TokenClass.RegExpLiteral] = 'regexp.ts';
tsTokenTypeTable[ts.TokenClass.StringLiteral] = 'string.ts';

const jsBracketTypeTable: INumberStringDictionary = Object.create(null);
jsBracketTypeTable['('.charCodeAt(0)] = 'delimiter.parenthesis.js';
jsBracketTypeTable[')'.charCodeAt(0)] = 'delimiter.parenthesis.js';
jsBracketTypeTable['{'.charCodeAt(0)] = 'delimiter.bracket.js';
jsBracketTypeTable['}'.charCodeAt(0)] = 'delimiter.bracket.js';
jsBracketTypeTable['['.charCodeAt(0)] = 'delimiter.array.js';
jsBracketTypeTable[']'.charCodeAt(0)] = 'delimiter.array.js';

const jsTokenTypeTable: INumberStringDictionary = Object.create(null);
jsTokenTypeTable[ts.TokenClass.Identifier] = 'identifier.js';
jsTokenTypeTable[ts.TokenClass.Keyword] = 'keyword.js';
jsTokenTypeTable[ts.TokenClass.Operator] = 'delimiter.js';
jsTokenTypeTable[ts.TokenClass.Punctuation] = 'delimiter.js';
jsTokenTypeTable[ts.TokenClass.NumberLiteral] = 'number.js';
jsTokenTypeTable[ts.TokenClass.RegExpLiteral] = 'regexp.js';
jsTokenTypeTable[ts.TokenClass.StringLiteral] = 'string.js';

function checkSheBang(deltaOffset: number, line: string, appendFn: (startIndex: number, type: string) => void): boolean {
    if (line.indexOf('#!') === 0) {
        appendFn(deltaOffset, 'comment.shebang');
        return true;
    }
    return false;
}
